from django.db import models
from admin_log import middleware


class AutoCreatedByField(models.ForeignKey):
    """ Automatically sets current admin user as object creator."""

    def __init__(self, to, on_delete, **kwargs):
        kwargs.setdefault('default', middleware.AdminLogMiddleware.get_user_id)
        kwargs.setdefault('related_name', '+')
        kwargs.setdefault('editable', False)
        kwargs.setdefault('blank', True)
        kwargs.setdefault('null', True)
        super().__init__(to, on_delete, **kwargs)


class AutoModifiedByField(AutoCreatedByField):
    """ Automatically sets current admin user as last object change author."""

    def pre_save(self, model_instance, add):
        value = middleware.AdminLogMiddleware.get_user_id()
        setattr(model_instance, self.attname, value)
        return value



from django.contrib import admin


READONLY_FIELDS = ('created', 'created_by', 'modified', 'modified_by')


class AdminLogMixin:
    def get_readonly_fields(self, request, obj=None):
        """ Adds logged fields to read only fields list."""
        # noinspection PyUnresolvedReferences
        fields = super().get_readonly_fields(request, obj)
        return tuple(fields) + READONLY_FIELDS


class LoggedModelAdmin(AdminLogMixin, admin.ModelAdmin):
    """ Model admin with read-only logged fields in details view."""



from typing import Optional

from django.http import HttpRequest


class AdminLogMiddleware:
    """
    Holds a singleton reference to current request and provides staff user id.
    """
    request: Optional[HttpRequest]

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        try:
            self.request_started(request)
            response = self.get_response(request)
        finally:
            self.request_finished(request)
        return response

    @classmethod
    def get_user_id(cls) -> Optional[int]:
        """
        Returns current user ID if user is authorized and marked as staff.
        """
        request = getattr(cls, 'request', None)
        if request is None:
            return None
        if request.user.is_authenticated and request.user.is_staff:
            return request.user.pk
        return None

    def request_started(self, request):
        self.__class__.request = request

    # noinspection PyUnusedLocal
    def request_finished(self, request):
        self.__class__.request = None



from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _
from model_utils.models import TimeStampedModel
from admin_log import fields


class AdminLogMixin(TimeStampedModel):
    """ Tracks author and date for creation and last modification."""

    created_by = fields.AutoCreatedByField(
        settings.AUTH_USER_MODEL, models.SET_NULL,
        verbose_name=_("Created by"))
    modified_by = fields.AutoModifiedByField(
        settings.AUTH_USER_MODEL, models.SET_NULL,
        verbose_name=_("Modified by"))

    class Meta:
        abstract = True
